(* Delphi components
   Display components for progress indication
   ==========================================
   
   Progress Meter 1.0 copyright 1996
     Mark Harwood - mark@temati.demon.co.uk

   clock style progress meter added: Jan. 1999
   bar style progress meter added: Okt. 2002
   animated image progress meter added: Feb. 2004
      J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   *)  

unit Prgrss;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

// Definiere CompPalPage (siehe Register)
{$Include UserComps.pas }

type
  TProgressStatus = (psOn, psOff, psNudge);

type
  TCustomProgress = class(TCustomControl)
  private
    { Private declarations }
    FProgressStatus    : TProgressStatus;
    FProgressTimer     : TTimer;
    FProgressForeColor : TColor;
    FProgressBackColor : TColor;
    FOnIncrement       : TNotifyEvent;
    FProgressInterval,
    FProgressCurrent,
    FIncr              : integer;
    procedure SetProgressStatus(value: TProgressStatus);
    procedure SetProgressInterval(value: integer);
    procedure SetProgressBackColor(value: TColor);
    procedure DrawStripe(i: integer);
    procedure SetOnIncrement(Value: TNotifyEvent);
  protected
    NudgeCounter : integer;
    function GetProgressForeColor : TColor;
    procedure SetProgressForeColor(value: TColor);
    procedure Paint; override;
    procedure TimerComplete(sender: TObject);
    property BackColor: TColor read FProgressBackColor write SetProgressBackColor default clRed;
    property ForeColor: TColor read GetProgressForeColor write SetProgressForeColor default clBlue;
    property Increment : integer read FIncr write FIncr default 5;
    property Interval: integer read FProgressInterval write SetProgressInterval default 100;
    property Status: TProgressStatus read FProgressStatus write SetProgressStatus default psOff;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Nudge;
    procedure Start;
    procedure Stop;
    procedure Pause (Paused : boolean);
  published
    property ShowHint;
    property Visible;
    property OnIncrement: TNotifyEvent read FOnIncrement write SetOnIncrement;
  end;

  TMDHProgress = class(TCustomProgress)
  published
    property ForeColor default clBlue;
    property BackColor default clRed;
    property Status;
    property Interval;
    end;

  TClkProgress = class(TCustomProgress)
  private
    Toggle : boolean;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Reset;
  published
    property ForeColor default clBlack;
    property BackColor default clWhite;
    property Status;
    property Interval;
  end;

  TBarProgress = class(TCustomProgress)
  private
    FBarWidth : integer;
    FBarFwd   : boolean;
    FTrans    : boolean;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Reset;
  published
    property BarColor : TColor read GetProgressForeColor write SetProgressForeColor default clNavy;
    property BackColor default clNavy;
    property Transparent : boolean read FTrans write FTrans default true;
    property BarWidth : integer read FBarWidth write FBarWidth default 10;
    property Increment;
    property Status;
    property Interval;
  end;

  TImageProgress = class(TCustomProgress)
  private
    FImgList   : TImageList;
    FOffImg    : boolean;
    procedure SetImgList (AImgList : TImageList);
    procedure SetOffImg (value : boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Reset;
  published
    property ShowOffImage : boolean read FOffImg write SetOffImg default false;
    property ImageList : TImageList read FimgList write SetImgList;
    property Status;
    property Interval;
  end;

procedure Register;

implementation

uses ImgList;

const
  Pi180 = Pi/180.0;

constructor TCustomProgress.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  { set defaults }
  Width:=150;
  Height:=14;
  FProgressForeColor:=clBlue;
  FProgressBackColor:=clRed;
  FProgressInterval:=100;
  FProgressStatus:=psOff;
  FProgressCurrent:=0;
  FIncr:=5;
  NudgeCounter:=0;
  end;

destructor TCustomProgress.Destroy;
begin
  if assigned(FProgressTimer) then FProgressTimer.Free;
  inherited Destroy;
end;

procedure TCustomProgress.DrawStripe(i: integer);
begin
  Canvas.Polygon([Point (i - Height, 1), Point(i, 1),
                  Point (i - Height, Height - 1), Point(i - (Height * 2), Height - 1)]);
end;

procedure TCustomProgress.Paint;
var
  x : integer;
begin
  if FProgressCurrent >= (Height * 2) then FProgressCurrent:=0;
  with Canvas do begin
    Brush.Color:=FProgressBackColor;
    Rectangle (0, 0, Width, Height);
    Brush.Color:=FProgressForeColor;
    Pen.Color:=FProgressForeColor;
    for x:=0 to (Width + Height) do
    begin
      if x mod (Height * 2)  = 0 then DrawStripe(x + FProgressCurrent);
    end; { for }
  end; { with }
end;

function TCustomProgress.GetProgressForeColor : TColor;
begin
  Result:=FProgressForeColor;
  end;

procedure TCustomProgress.SetProgressForeColor(value: TColor);
begin
  FProgressForeColor:=value;
  invalidate;
end;

procedure TCustomProgress.SetProgressBackColor(value: TColor);
begin
  FProgressBackColor:= value;
  invalidate
end;

procedure TCustomProgress.SetProgressStatus(value: TProgressStatus);
begin
  if value <> FProgressStatus then begin
    if assigned(FProgressTimer) then begin
      FProgressTimer.Free;
      FProgressTimer:=nil;
      end;
    FProgressStatus:=value;
    if FProgressStatus = psOn then begin
      FProgressTimer:=TTimer.Create(Self);
      FProgressTimer.Interval:=FProgressInterval;
      FProgressTimer.OnTimer:=TimerComplete;
      end;
    if FProgressStatus = psNudge then NudgeCounter:=0;
    Invalidate;
    end;
  end;

procedure TCustomProgress.SetProgressInterval(value: integer);
begin
  if value <> FProgressInterval then
  begin
    FProgressInterval:=value;
    if assigned(FProgressTimer) then FProgressTimer.Interval:=FProgressInterval;
  end;
end;

procedure TCustomProgress.TimerComplete(sender: TObject);
begin
  FProgressCurrent:=FProgressCurrent + FIncr;
  if Assigned(FOnIncrement) then FOnIncrement(Self);
  Invalidate;
  end;

procedure TCustomProgress.SetOnIncrement(Value: TNotifyEvent);
begin
  FOnIncrement:=Value;
  end;

procedure TCustomProgress.Nudge;
begin
  if FProgressStatus<>psNudge then exit;
  inc (NudgeCounter);
  if NudgeCounter>FProgressInterval then begin
    TimerComplete (Self);
    NudgeCounter:=0;
  end;
end;

procedure TCustomProgress.Start;
begin
  Status:=psOn;
  end;

procedure TCustomProgress.Stop;
begin
  Status:=psOff;
  end;

procedure TCustomProgress.Pause (Paused : boolean);
begin
  if assigned(FProgressTimer) then FProgressTimer.Enabled:=not Paused;
  end;

{ ------------------------------------------------------------------- }
(* runde uhrenartige Fortschrittsanzeige *)
constructor TClkProgress.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  Toggle:=true;
  Width:=26;
  Height:=26;
  FProgressForeColor:=clBlack;
  FProgressBackColor:=clWhite;
  end;

procedure TClkProgress.Reset;
begin
  FProgressCurrent:=0;
  Toggle:=true;
  Invalidate;
  end;

procedure TClkProgress.Paint;
var
  x,y,r,h1,h2 : integer;
begin
  if FProgressCurrent >= 360 then begin
    FProgressCurrent:=0;
    Toggle:=not Toggle;
    end;
  with Canvas do begin
    Pen.Color:=clBtnShadow; Brush.Color:=clBtnShadow;
    Ellipse (0,0,Height-1,Height-1);
    Brush.Color:=clBtnHighlight; Pen.Color:=clBtnHighlight;
    Ellipse (1,1,Height,Height);
    Pen.Color:=clBtnFace;
    if Toggle then Brush.Color:=FProgressForeColor
    else Brush.Color:=FProgressBackColor;
    h1:=1; h2:=Height-1;
    Ellipse (h1,h1,h2,h2);
    if Toggle then Brush.Color:=FProgressBackColor
    else Brush.Color:=FProgressForeColor;
    r:=pred(Height div 2);
    x:=r+round(r*sin(Pi180*FProgressCurrent));
    y:=r-round(r*cos(Pi180*FProgressCurrent));
    Pie (h1,h1,h2,h2,r,0,x,y);
    end; { with }
  end;

{ ------------------------------------------------------------------- }
(* balkenartige Fortschrittsanzeige *)
constructor TBarProgress.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width:=151; Height:=11; FTrans:=true;
  FProgressForeColor:=clNavy;
  FProgressBackColor:=clBtnFace;
  ParentColor:=FTrans;
  FIncr:=5;
  FBarWidth:=10;
  FBarFwd:=true;
  end;

procedure TBarProgress.Reset;
begin
  FProgressCurrent:=0;
  FBarFwd:=true;
  Invalidate;
  end;

procedure TBarProgress.Paint;
var
  x : integer;
begin
  if FProgressCurrent>=Width-FBarWidth-2 then begin
    FProgressCurrent:=0;
    FBarFwd:=not FBarFwd;    // toggle between forward and reverse movement
    end;
  with Canvas do begin
    if FTrans then Brush.Color:=Color
    else Brush.Color:=FProgressBackColor;
    with Pen do begin
      Color:=clBtnShadow;
      Width:=1;
      end;
    Rectangle(0,0,Width,Height);
    Pen.Color:=clBtnHighlight;
    MoveTo (Width-1,0); LineTo (Width-1,Height-1); LineTo (0,Height-1);
    Brush.Color:=FProgressForeColor;
    if FBarFwd then x:=FProgressCurrent+2 else x:=Width-FBarWidth-2-FProgressCurrent;
    FillRect (Rect(x,2,x+FBarWidth,Height-2));
    end;
  end;

{ ------------------------------------------------------------------- }
(* Fortschrittsanzeige mit animierten Bildern*)
constructor TImageProgress.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  FImgList:=nil;
  Width:=128;
  Height:=32;
  FIncr:=1;
  end;

procedure TImageProgress.SetImgList (AImgList : TImageList);
begin
  Width:=Width+1; Paint;
  FImgList:=AImgList;
  if assigned(FImgList) then begin
    Width:=FImgList.Width;
    Height:=FImgList.Height;
    end;
  Reset;
  end;

procedure TImageProgress.SetOffImg (value : boolean);
begin
  FOffImg:=Value;
  Invalidate;
  end;

procedure TImageProgress.Reset;
begin
  FProgressCurrent:=0;
  Invalidate;
  end;

procedure TImageProgress.Paint;
var
  n : integer;
begin
  if assigned(FImgList) then begin
    if FOffImg then begin
      if Status=psOff then n:=0
      else n:=FProgressCurrent mod (FImgList.Count-1) +1;
      end
    else n:=FProgressCurrent mod FImgList.Count;
    Width:=FImgList.Width; Height:=FImgList.Height;
    FImgList.Draw(Canvas,0,0,n,dsTransparent,itImage);
    end
  else with Canvas do begin
    Brush.Color:=clWhite;
    FillRect(Rect(0,0,Width,Height));
    Brush.Color:=clRed;
    FillRect(Rect(0,0,FProgressCurrent mod Width,Height));
    end;
  end;

{ ---------------------------------------------------------------- }
procedure Register;
begin
  RegisterComponents(CompPalPage, [TMDHProgress]);
  RegisterComponents(CompPalPage, [TClkProgress]);
  RegisterComponents(CompPalPage, [TBarProgress]);
  RegisterComponents(CompPalPage, [TImageProgress]);
  end;

end.
